"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
"""

# -*- coding: utf-8 -*-
from libs import base
from constants import *
from helpers.info_dict import InfoDict


class Namespace(object):
    """Namespace contains all information about namespace"""

    def __new__(cls, name, ip, port, cluster_id):

        #if base.is_node_available(ip, port) is False:
        #    return None
        return super(Namespace, cls).__new__(cls, name, ip, port, cluster_id)

    def __init__(self, name, ip, port, cluster_id):

        self._name = name
        self._ip = ip
        self._port = port
        self._address = "%s:%s" % (ip, port)
        self._cluster_id = cluster_id

        self._statistics = {}
        self._modified_statistics = {}
        self._index_statistics = {}
        self._modified_index_statistics = {}
        self.configuration_params = ["type", "file", "filesize",
            "load-at-startup", "data-in-memory", "defrag-period",
            "defrag-lwm-pct", "defrag-max-blocks", "defrag-startup-minimum",
            "write-block-size", "ticker-interval", "replication-factor",
            "low-water-pct", "high-water-memory-pct", "high-water-disk-pct",
            "high-water-pct", "evict-pct", "stop-writes-pct", "memory-size",
            "default-ttl", "max-ttl", "allow-versions", "single-bin",
            "node_status"]

    def get_configurations(self):

        return self.get_subset_of_statistics(self.configuration_params)

    def get_configurations_without_ns(self):

        configs_without_ns = InfoDict()

        stats = self.get_statistics()
        for attr in self.configuration_params:
            value = stats.get(attr, None)
            if value is not None:
                configs_without_ns[attr] = value
        return configs_without_ns

    def get_subset_of_configurations(self, attrs=[]):

        subset = InfoDict()
        configurations = self.get_configurations()

        for attr in attrs:
            subset[attr] = configurations.get(attr, NOT_SUPPORTED)
        return subset

    def update_statistics(self):

        statistics = base.get_namespace_statistics(self._name, self._ip, self._port, clear_cache=True)
        self._modified_statistics = dict(set(statistics.items()) - set(self._statistics.items()))
        self._statistics = statistics

    def get_statistics(self):

        return self._statistics

    def get_modified_statistics(self):

        self.update_statistics()
        return self._modified_statistics

    def get_updated_statistics(self):

        statistics = base.get_namespace_statistics(self._name,
            self._ip, self._port)
        self._modified_statistics = set(statistics.items()) - set(self._statistics.items())
        self._statistics = statistics

        ##    * Unit test cases *
        ##from random import randrange

        ##free_pct_disk = ["2", "5", "10"]
        ##self._statistics['free-pct-disk'] = free_pct_disk[randrange(0, 3)]

        ##available_pct = ["5", "10", "20"]
        ##self._statistics["available_pct"] = available_pct[randrange(0, 3)]

        ##disks = ["80", "90", "100"]
        ##self._statistics["used-disk-pct"] = disks[randrange(0, 3)]
        ##self._statistics["high-water-disk-pct"] = "80"
        ##self._statistics["stop-writes-pct"] = "90"

        ##memory = ["80", "90", "100"]
        ##self._statistics["used-memory-pct"] = memory[randrange(0, 3)]
        ##self._statistics["high-water-memory-pct"] = "80"
        ##self._statistics["stop-writes-pct"] = "90"

        return self._statistics

    def get_subset_of_statistics(self, attrs=[]):

        subset = InfoDict()

        stats = self.get_statistics()
        for attr in attrs:
            subset[attr] = stats.get(attr, NOT_SUPPORTED)
        return subset

    def get_statistic(self, attr):

        stats = self.get_statistics()
        return stats.get(attr, NOT_SUPPORTED)

    def get_disk(self):

        return self.get_statistic("used-bytes-disk"), self.get_statistic("total-bytes-disk")

    def get_disk_stats(self):

        attrs = ["used-bytes-disk", "total-bytes-disk"]
        stats = self.get_subset_of_statistics(attrs)
        try:
            stats["used-bytes-disk"] = int(stats["used-bytes-disk"])
            stats["total-bytes-disk"] = int(stats["total-bytes-disk"])
            stats["free-bytes-disk"] = stats["total-bytes-disk"] - stats["used-bytes-disk"]
        except:
            stats["free-bytes-disk"] = NOT_SUPPORTED
        return stats

    def get_memory_stats(self):

        attrs = ["used-bytes-memory", "total-bytes-memory"]
        stats = self.get_subset_of_statistics(attrs)
        try:
            stats["used-bytes-memory"] = int(stats["used-bytes-memory"])
            stats["total-bytes-memory"] = int(stats["total-bytes-memory"])
            stats["free-bytes-memory"] = stats["total-bytes-memory"] - stats["used-bytes-memory"]
        except:
            stats["free-bytes-memory"] = NOT_SUPPORTED
        return stats

    def get_memory(self):

        return self.get_statistic("used-bytes-memory"), self.get_statistic("total-bytes-memory")

    def get_sets_info(self):

        self._sets_info = base.get_namespace_sets_info(self._name)
        return self._sets_info

    def get_index_statistics(self, index_name, stats_type="all"):

        index_statistics = base.get_index_stats(self._ip, self._port, self._name, index_name)
        self._modified_index_statistics = dict(set(index_statistics.items()) - set(self._index_statistics.items()))
        self._index_statistics = index_statistics
        return self._index_statistics if stats_type == "all" else self._modified_index_statistics
